"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var ClusterService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClusterService = void 0;
const common_1 = require("@nestjs/common");
const democracy_1 = __importDefault(require("democracy"));
const lodash_1 = __importDefault(require("lodash"));
const os = __importStar(require("os"));
const config_service_1 = require("../config/config.service");
const id_1 = require("../util/id");
const resolvers_1 = require("./resolvers");
let mdns;
try {
    mdns = require('mdns');
}
catch (e) {
    common_1.Logger.error(e.message, e.stack, 'ClusterService');
    mdns = undefined;
}
let ClusterService = ClusterService_1 = class ClusterService extends democracy_1.default {
    constructor(configService) {
        const config = configService.get('cluster');
        const globalConfig = configService.get('global');
        const networkInterfaces = lodash_1.default.flatMap(config.networkInterface
            ? os.networkInterfaces()[config.networkInterface]
            : os.networkInterfaces());
        const ip = networkInterfaces.find((address) => address.internal === false && address.family === 'IPv4').address;
        super({
            id: (0, id_1.makeId)(globalConfig.instanceName),
            source: `${ip}:${config.port}`,
            peers: Array.from(config.peerAddresses),
            timeout: config.timeout * 1000,
            weight: config.weight,
        });
        this.networkInterfaces = networkInterfaces;
        this.configService = configService;
        this.config = config;
        this.logger = new common_1.Logger(ClusterService_1.name);
    }
    onModuleInit() {
        this.on('added', this.handleNodeAdded);
        this.on('removed', this.handleNodeRemoved);
        this.on('elected', this.handleNodeElected);
        this.on('leader', this.handleNodeElected);
        this.on('peers', this.addMissingPeers);
    }
    onApplicationBootstrap() {
        if (this.config.autoDiscovery) {
            if (mdns !== undefined) {
                try {
                    this.startBonjourDiscovery();
                }
                catch (e) {
                    this.logger.error(`Failed to start mdns discovery (${e.message})`, e.stack);
                }
            }
            else {
                this.logger.warn('Dependency "mdns" was not found, automatic discovery has been disabled. You will have to provide the addresses of other room-assistant nodes manually in the config.');
            }
        }
        this.broadcastPeers();
    }
    onApplicationShutdown() {
        var _a, _b, _c;
        (_a = this.clusterAdvertisement) === null || _a === void 0 ? void 0 : _a.stop();
        (_b = this.apiAdvertisement) === null || _b === void 0 ? void 0 : _b.stop();
        (_c = this.browser) === null || _c === void 0 ? void 0 : _c.stop();
    }
    isMajorityLeader() {
        return this.isLeader() && this.quorumReached();
    }
    quorumReached() {
        const activeNodes = Object.values(this.nodes()).filter((node) => (node === null || node === void 0 ? void 0 : node.state) !== 'removed');
        return !this.config.quorum || activeNodes.length >= this.config.quorum;
    }
    broadcastPeers() {
        var _a;
        if ((_a = this.options) === null || _a === void 0 ? void 0 : _a.peers) {
            this.send('peers', this.options.peers);
        }
    }
    addMissingPeers(peers) {
        peers.forEach((peer) => {
            const index = this.options.peers.findIndex((p) => p[0] === peer[0] && p[1] === peer[1]);
            if (index < 0) {
                this.options.peers.push(peer);
            }
        });
    }
    processEvent(msg) {
        super.processEvent(msg);
        const data = this.decodeMsg(msg);
        if (!data.chunk && data.state === 'leader') {
            const leaders = Object.entries(this._nodes).filter((node) => { var _a; return ((_a = node[1]) === null || _a === void 0 ? void 0 : _a.state) === 'leader'; });
            if (leaders.length > 1) {
                leaders.forEach((leader) => {
                    this._nodes[leader[0]].state = 'citizen';
                });
                this.holdElections();
            }
        }
        return this;
    }
    checkBallots(candidate) {
        super.checkBallots(candidate);
        const node = this._nodes[candidate];
        if (this.config.peerAddresses.includes(node.source)) {
            this.logger.debug(`Saving configured peer ${node.source} from ultimate removal`);
            clearTimeout(node.disconnected);
            delete node.disconnected;
        }
        return this;
    }
    startBonjourDiscovery() {
        this.clusterAdvertisement = mdns.createAdvertisement(mdns.udp('room-assistant'), this.config.port, {
            networkInterface: this.config.networkInterface,
        });
        this.apiAdvertisement = mdns.createAdvertisement(mdns.tcp('room-asst-api'), this.configService.get('global').apiPort, {
            networkInterface: this.config.networkInterface,
        });
        const defaultGetAddr = 'DNSServiceGetAddrInfo' in mdns.dns_sd
            ? mdns.rst.DNSServiceGetAddrInfo()
            : mdns.rst.getaddrinfo({ families: [0] });
        const sequence = [
            mdns.rst.DNSServiceResolve(),
            process.env.NODE_DIG_RESOLVER ? resolvers_1.getAddrInfoDig : defaultGetAddr,
            mdns.rst.makeAddressesUnique(),
        ];
        this.browser = mdns.createBrowser(mdns.udp('room-assistant'), {
            resolverSequence: sequence,
        });
        this.browser.on('serviceUp', this.handleNodeDiscovery.bind(this));
        this.browser.on('error', (e) => {
            this.logger.error(e.message, e.trace);
        });
        this.logger.log('Starting mDNS advertisements and discovery');
        this.clusterAdvertisement.start();
        this.apiAdvertisement.start();
        this.browser.start();
    }
    handleNodeDiscovery(service) {
        const ownIps = this.networkInterfaces.map((info) => info.address);
        if (lodash_1.default.some(service.addresses, (address) => ownIps.includes(address)) ||
            lodash_1.default.some(this.options.peers, (peer) => service.addresses.some((address) => peer[0] === address && peer[1] === service.port.toString()))) {
            return;
        }
        this.options.peers.push([service.addresses[0], service.port.toString()]);
        this.send('hello');
    }
    handleNodeAdded(node) {
        this.logger.log(`Added ${node.source} to the cluster with id ${node.id}`);
        this.broadcastPeers();
    }
    handleNodeRemoved(node) {
        this.logger.log(`Removed ${node.source} from the cluster with id ${node.id}`);
    }
    handleNodeElected(node) {
        this.logger.log(`${node.id} has been elected as leader`);
    }
};
ClusterService = ClusterService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [config_service_1.ConfigService])
], ClusterService);
exports.ClusterService = ClusterService;
//# sourceMappingURL=cluster.service.js.map