"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var EntitiesService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntitiesService = void 0;
const common_1 = require("@nestjs/common");
const entity_proxy_1 = require("./entity.proxy");
const nest_emitter_1 = require("nest-emitter");
const cluster_service_1 = require("../cluster/cluster.service");
const config_service_1 = require("../config/config.service");
const debounce_proxy_1 = require("./debounce.proxy");
const rolling_average_proxy_1 = require("./rolling-average.proxy");
let EntitiesService = EntitiesService_1 = class EntitiesService {
    constructor(clusterService, configService, emitter) {
        this.clusterService = clusterService;
        this.configService = configService;
        this.emitter = emitter;
        this.entities = new Map();
        this.config = configService.get('entities');
        this.logger = new common_1.Logger(EntitiesService_1.name);
    }
    has(id) {
        return this.entities.has(id);
    }
    get(id) {
        return this.entities.get(id);
    }
    getAll() {
        return Array.from(this.entities.values());
    }
    add(entity, customizations) {
        if (this.entities.has(entity.id)) {
            throw new Error(`Entity with id ${entity.id} already exists!`);
        }
        this.logger.debug(`Adding new entity ${entity.id}`);
        let proxy = new Proxy(entity, new entity_proxy_1.EntityProxyHandler(this.emitter, this.hasAuthorityOver.bind(this)));
        proxy = this.applyEntityBehaviors(proxy);
        this.entities.set(entity.id, proxy);
        this.emitter.emit('newEntity', proxy, customizations);
        return proxy;
    }
    hasAuthorityOver(entity) {
        return (!entity.distributed ||
            !entity.stateLocked ||
            this.clusterService.isMajorityLeader());
    }
    applyEntityBehaviors(entity) {
        var _a, _b;
        const behaviorConfig = this.config.behaviors[entity.id];
        let proxy = entity;
        if ((_a = behaviorConfig === null || behaviorConfig === void 0 ? void 0 : behaviorConfig.rollingAverage) === null || _a === void 0 ? void 0 : _a.window) {
            proxy = new Proxy(proxy, new rolling_average_proxy_1.RollingAverageProxyHandler(behaviorConfig.rollingAverage));
        }
        if ((_b = behaviorConfig === null || behaviorConfig === void 0 ? void 0 : behaviorConfig.debounce) === null || _b === void 0 ? void 0 : _b.wait) {
            proxy = new Proxy(proxy, new debounce_proxy_1.DebounceProxyHandler(behaviorConfig.debounce));
        }
        return proxy;
    }
};
EntitiesService = EntitiesService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(2, (0, nest_emitter_1.InjectEventEmitter)()),
    __metadata("design:paramtypes", [cluster_service_1.ClusterService,
        config_service_1.ConfigService, Object])
], EntitiesService);
exports.EntitiesService = EntitiesService;
//# sourceMappingURL=entities.service.js.map