"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityProxyHandler = void 0;
class EntityProxyHandler {
    constructor(emitter, hasAuthorityOver) {
        this.emitter = emitter;
        this.hasAuthorityOver = hasAuthorityOver;
        this.diff = [];
    }
    get(target, p) {
        if (typeof target[p] === 'object' &&
            target[p] !== null &&
            !(target[p] instanceof Date)) {
            return new Proxy(target[p], new EntityPropertyProxyHandler(`/${p.toString()}`, (diff) => this.emitEntityUpdate(target, diff)));
        }
        else {
            return target[p];
        }
    }
    set(target, p, value) {
        const oldValue = target[p];
        target[p] = value;
        if (oldValue !== value) {
            this.emitEntityUpdate(target, [
                { path: `/${p.toString()}`, oldValue, newValue: value },
            ]);
        }
        return true;
    }
    emitEntityUpdate(entity, diff) {
        this.target = entity;
        this.diff.push(...diff);
        if (!this.flushTimeout) {
            this.flushTimeout = setTimeout(this.flushEntityUpdates.bind(this), 100);
        }
    }
    flushEntityUpdates() {
        this.emitter.emit('entityUpdate', this.target, this.diff, this.hasAuthorityOver(this.target));
        this.diff = [];
        this.flushTimeout = undefined;
    }
}
exports.EntityProxyHandler = EntityProxyHandler;
class EntityPropertyProxyHandler {
    constructor(path, emitterFunc) {
        this.path = path;
        this.emitterFunc = emitterFunc;
    }
    get(target, p) {
        if (typeof target[p] === 'object' &&
            target[p] !== null &&
            !(target[p] instanceof Date)) {
            return new Proxy(target[p], new EntityPropertyProxyHandler(`${this.path}/${p.toString()}`, this.emitterFunc));
        }
        else {
            return target[p];
        }
    }
    set(target, p, value) {
        const oldValue = target[p];
        target[p] = value;
        if (oldValue !== value) {
            this.emitterFunc([
                { path: `${this.path}/${p.toString()}`, oldValue, newValue: value },
            ]);
        }
        return true;
    }
}
//# sourceMappingURL=entity.proxy.js.map