"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RollingAverageProxyHandler = void 0;
const lodash_1 = __importDefault(require("lodash"));
class RollingAverageProxyHandler {
    constructor(config) {
        this.config = config;
        this.values = [];
    }
    set(target, p, value) {
        if (p == 'state') {
            this.values.push(new TimedValue(value));
            this.updateTargetState(target);
            if (this.updateInterval === undefined) {
                setInterval(() => this.updateTargetState(target), 1000);
            }
        }
        else {
            target[p] = value;
        }
        return true;
    }
    updateTargetState(target) {
        this.values = this.values.filter((value, index, array) => {
            var _a;
            return (value.msAgo <= this.config.window * 1000 ||
                ((_a = array[index + 1]) === null || _a === void 0 ? void 0 : _a.msAgo) <= this.config.window * 1000);
        });
        if (this.values.length > 0) {
            const weights = this.values.reduce((map, value, index, array) => {
                var _a;
                const weight = lodash_1.default.clamp(value.msAgo, this.config.window * 1000) -
                    (((_a = array[index + 1]) === null || _a === void 0 ? void 0 : _a.msAgo) || 0);
                if (map.has(value.value)) {
                    map.set(value.value, map.get(value.value) + weight);
                }
                else {
                    map.set(value.value, weight);
                }
                return map;
            }, new Map());
            if (Array.from(weights.keys()).every((x) => typeof x === 'number')) {
                const [weightedValueSum, weightSum] = Array.from(weights.entries()).reduce((previous, value) => {
                    return [previous[0] + value[0] * value[1], previous[1] + value[1]];
                }, [0, 0]);
                target.state = weightSum > 0 ? weightedValueSum / weightSum : undefined;
            }
            else {
                const winner = Array.from(weights.entries()).reduce((previous, value) => {
                    return previous[0] == undefined || value[1] > previous[1]
                        ? value
                        : previous;
                }, [undefined, 0]);
                target.state = winner[0];
            }
        }
    }
}
exports.RollingAverageProxyHandler = RollingAverageProxyHandler;
class TimedValue {
    constructor(value) {
        this.createdAt = new Date();
        this.value = value;
    }
    get msAgo() {
        return Date.now() - this.createdAt.getTime();
    }
}
//# sourceMappingURL=rolling-average.proxy.js.map