/// <reference types="node" />
import { OnApplicationShutdown } from '@nestjs/common';
import { Peripheral } from '@mkerix/noble';
import { ConfigService } from '../../config/config.service';
import { Device } from '../../integrations/bluetooth-classic/device';
import { Counter } from 'prom-client';
export declare class BluetoothService implements OnApplicationShutdown {
    private readonly configService;
    private readonly advertisementReceivedCounter;
    private readonly logger;
    private readonly classicConfig;
    private readonly bleConfig;
    private readonly adapters;
    private _lowEnergyAdapterId;
    private _successiveErrorsOccurred;
    private lastLowEnergyDiscovery;
    private scanStartedAt?;
    private queryMutexAvailable;
    constructor(configService: ConfigService, advertisementReceivedCounter: Counter<string>);
    get timeSinceLastDiscovery(): number | null;
    get lowEnergyAdapterId(): number | undefined;
    get successiveErrorsOccurred(): number;
    onApplicationShutdown(): void;
    onLowEnergyDiscovery(callback: (peripheral: Peripheral) => void): void;
    connectLowEnergyDevice(peripheral: Peripheral): Promise<Peripheral>;
    disconnectLowEnergyDevice(peripheral: Peripheral): Promise<void>;
    acquireQueryMutex(): boolean;
    releaseQueryMutex(): void;
    queryLowEnergyDevice(target: Peripheral, serviceUuid: string, characteristicUuid: string): Promise<Buffer | null>;
    private readLowEnergyCharacteristic;
    inquireClassicRssi(adapterId: number, address: string): Promise<number>;
    cancelClassicInquiry(adapterId: number): Promise<void>;
    inquireClassicDeviceInfo(adapterId: number, address: string): Promise<Device>;
    resetHciDevice(adapterId: number): Promise<void>;
    lockAdapter(adapterId: number): void;
    unlockAdapter(adapterId: number): Promise<void>;
    unlockDeadlockedAdapters(): void;
    verifyLowEnergyScanner(): Promise<void>;
    private setupNoble;
    private handleScanStart;
    private handleScanStop;
    private handleDiscover;
    private handleAdapterStateChange;
    private connectLowEnergyDeviceWithRetry;
}
