"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var BluetoothService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BluetoothService = void 0;
const common_1 = require("@nestjs/common");
const noble_1 = __importDefault(require("@mkerix/noble"));
const util_1 = __importDefault(require("util"));
const child_process_1 = require("child_process");
const config_service_1 = require("../../config/config.service");
const promises_1 = require("../../util/promises");
const schedule_1 = require("@nestjs/schedule");
const lodash_1 = __importDefault(require("lodash"));
const prom_client_1 = require("prom-client");
const nestjs_prometheus_1 = require("@willsoto/nestjs-prometheus");
const RSSI_REGEX = new RegExp(/-?[0-9]+/);
const INQUIRY_LOCK_TIMEOUT = 30 * 1000;
const SCAN_NO_PERIPHERAL_TIMEOUT = 30 * 1000;
const BLE_CONNECTION_RETRIES = 5;
const BLE_CONNECTION_TIMEOUT = 10 * 1000;
const BLE_READ_TIMEOUT = 15 * 1000;
const execPromise = util_1.default.promisify(child_process_1.exec);
class BluetoothAdapter {
}
class BluetoothAdapterMap extends Map {
    getState(key) {
        var _a;
        return (_a = this.get(key)) === null || _a === void 0 ? void 0 : _a.state;
    }
    setState(key, state) {
        return this.set(key, { state, startedAt: new Date() });
    }
}
let BluetoothService = BluetoothService_1 = class BluetoothService {
    constructor(configService, advertisementReceivedCounter) {
        this.configService = configService;
        this.advertisementReceivedCounter = advertisementReceivedCounter;
        this.logger = new common_1.Logger(BluetoothService_1.name);
        this.adapters = new BluetoothAdapterMap();
        this._successiveErrorsOccurred = 0;
        this.queryMutexAvailable = true;
        this.classicConfig = this.configService.get('bluetoothClassic');
        this.bleConfig = this.configService.get('bluetoothLowEnergy');
    }
    get timeSinceLastDiscovery() {
        if (this._lowEnergyAdapterId == null ||
            !['inactive', 'scan'].includes(this.adapters.getState(this._lowEnergyAdapterId))) {
            return null;
        }
        const timestamp = this.lastLowEnergyDiscovery == null
            ? this.adapters.get(this._lowEnergyAdapterId).startedAt
            : this.lastLowEnergyDiscovery;
        return Date.now() - timestamp.getTime();
    }
    get lowEnergyAdapterId() {
        return this._lowEnergyAdapterId;
    }
    get successiveErrorsOccurred() {
        return this._successiveErrorsOccurred;
    }
    onApplicationShutdown() {
        if (this._lowEnergyAdapterId != null &&
            this.adapters.getState(this._lowEnergyAdapterId) != 'inquiry') {
            noble_1.default.stopScanning();
        }
    }
    onLowEnergyDiscovery(callback) {
        if (this._lowEnergyAdapterId == undefined) {
            this.setupNoble();
        }
        noble_1.default.on('discover', callback);
    }
    async connectLowEnergyDevice(peripheral) {
        if (!peripheral.connectable) {
            throw new Error('Trying to connect to a non-connectable device');
        }
        if (peripheral.state === 'connected') {
            return peripheral;
        }
        else if (peripheral.state === 'connecting') {
            throw new Error(`Connection to ${peripheral.address} is already trying to be established`);
        }
        this.lockAdapter(this._lowEnergyAdapterId);
        try {
            await this.connectLowEnergyDeviceWithRetry(peripheral, BLE_CONNECTION_RETRIES);
            return peripheral;
        }
        catch (e) {
            this.logger.error(`Failed to connect to ${peripheral.address}: ${e.message}`, e.trace);
            await this.unlockAdapter(this._lowEnergyAdapterId);
            throw e;
        }
    }
    async disconnectLowEnergyDevice(peripheral) {
        if (!['connecting', 'connected'].includes(peripheral.state)) {
            return;
        }
        this.logger.debug(`Disconnecting from BLE device at address ${peripheral.address}`);
        try {
            await (0, promises_1.promiseWithTimeout)(peripheral.disconnectAsync(), 1000);
        }
        catch (e) {
            this.logger.debug(`Failed to disconnect from ${peripheral.address}: ${e.message}`, e.trace);
            await this.resetHciDevice(this._lowEnergyAdapterId);
        }
    }
    acquireQueryMutex() {
        if (this.queryMutexAvailable) {
            this.queryMutexAvailable = false;
            return true;
        }
        return false;
    }
    releaseQueryMutex() {
        this.queryMutexAvailable = true;
    }
    async queryLowEnergyDevice(target, serviceUuid, characteristicUuid) {
        if (this.queryMutexAvailable) {
            this.logger.error(`Permission to query ${target.id} has not been acquired`);
            return null;
        }
        const peripheral = await this.connectLowEnergyDevice(target);
        try {
            return await this.readLowEnergyCharacteristic(peripheral, serviceUuid, characteristicUuid);
        }
        catch (e) {
            this.logger.error(`Failed to query value from ${target.id}: ${e.message}`, e.trace);
            if (e.message === 'timed out') {
                await this.resetHciDevice(this.lowEnergyAdapterId);
            }
            return null;
        }
        finally {
            if (!['disconnecting', 'disconnected'].includes(target.state)) {
                await this.disconnectLowEnergyDevice(target);
            }
            await this.unlockAdapter(this._lowEnergyAdapterId);
        }
    }
    async readLowEnergyCharacteristic(peripheral, serviceUuid, characteristicUuid) {
        if (peripheral.state !== 'connected') {
            return null;
        }
        const cutoffTime = Date.now() + BLE_READ_TIMEOUT;
        let timeout = cutoffTime - Date.now();
        const services = (await (0, promises_1.promiseWithTimeout)(peripheral.discoverServicesAsync([serviceUuid]), timeout));
        timeout = cutoffTime - Date.now();
        if (services.length > 0 &&
            peripheral.state === 'connected' &&
            timeout > 0) {
            const characteristics = (await (0, promises_1.promiseWithTimeout)(services[0].discoverCharacteristicsAsync([characteristicUuid]), timeout));
            timeout = cutoffTime - Date.now();
            if (characteristics.length > 0 &&
                peripheral.state === 'connected' &&
                timeout > 0) {
                return await (0, promises_1.promiseWithTimeout)(characteristics[0].readAsync(), timeout);
            }
        }
        return null;
    }
    async inquireClassicRssi(adapterId, address) {
        var _a, _b;
        this.lockAdapter(adapterId);
        this.logger.debug(`Querying for RSSI of ${address} using hcitool`);
        try {
            const output = await (0, promises_1.promiseWithTimeout)(execPromise(`hcitool -i hci${adapterId} cc "${address}" && hcitool -i hci${adapterId} rssi "${address}"`, {
                timeout: this.classicConfig.scanTimeLimit * 1000,
                killSignal: 'SIGKILL',
            }), this.classicConfig.scanTimeLimit * 1000 * 2);
            const matches = output.stdout.match(RSSI_REGEX);
            this._successiveErrorsOccurred = 0;
            return (matches === null || matches === void 0 ? void 0 : matches.length) > 0 ? parseInt(matches[0], 10) : undefined;
        }
        catch (e) {
            if (e.signal === 'SIGKILL') {
                this.logger.debug(`Query of ${address} reached scan time limit, cancelling connection attempt`);
                await this.cancelClassicInquiry(adapterId);
            }
            else if (((_a = e.message) === null || _a === void 0 ? void 0 : _a.includes('Input/output')) ||
                ((_b = e.message) === null || _b === void 0 ? void 0 : _b.includes('I/O'))) {
                this.logger.debug(e.message);
            }
            else {
                this.logger.error(`Inquiring RSSI via BT Classic failed: ${e.message}`);
                this._successiveErrorsOccurred++;
            }
            return undefined;
        }
        finally {
            this.unlockAdapter(adapterId);
        }
    }
    async cancelClassicInquiry(adapterId) {
        await execPromise(`hcitool -i hci${adapterId} cmd 0x01 0x0008`, {
            timeout: 3000,
        });
    }
    async inquireClassicDeviceInfo(adapterId, address) {
        this.lockAdapter(adapterId);
        try {
            const output = await (0, promises_1.promiseWithTimeout)(execPromise(`hcitool -i hci${adapterId} info "${address}"`, {
                timeout: this.classicConfig.scanTimeLimit * 1000,
            }), 6000);
            const nameMatches = /Device Name: (.+)/g.exec(output.stdout);
            const manufacturerMatches = /OUI Company: (.+) \(.+\)/g.exec(output.stdout);
            return {
                address,
                name: nameMatches ? nameMatches[1] : address,
                manufacturer: manufacturerMatches ? manufacturerMatches[1] : undefined,
            };
        }
        catch (e) {
            this.logger.error(e.message, e.stack);
            return {
                address,
                name: address,
            };
        }
        finally {
            this.unlockAdapter(adapterId);
        }
    }
    async resetHciDevice(adapterId) {
        if (this.adapters.getState(adapterId) === 'resetting') {
            throw new Error('Adapter is already resetting');
        }
        this.logger.debug(`Resetting HCI adapter ${adapterId}`);
        this.adapters.setState(adapterId, 'resetting');
        if (this._lowEnergyAdapterId === adapterId) {
            noble_1.default.stopScanning();
        }
        try {
            await execPromise(`hciconfig hci${adapterId} reset`, {
                timeout: 3000,
            });
        }
        catch (e) {
            this.logger.error(e.message);
        }
        if (this._lowEnergyAdapterId === adapterId) {
            await (0, promises_1.sleep)(5000);
            try {
                noble_1.default.resetBindings();
                await this.handleAdapterStateChange(noble_1.default.state);
            }
            catch (e) {
                this.logger.error('Failed to reset low energy library', e.stack);
            }
            finally {
                if (this.adapters.getState(this._lowEnergyAdapterId) === 'resetting') {
                    this.adapters.setState(this._lowEnergyAdapterId, 'inactive');
                }
            }
        }
        else {
            this.adapters.setState(adapterId, 'inactive');
        }
    }
    lockAdapter(adapterId) {
        this.logger.debug(`Locking adapter ${adapterId}`);
        switch (this.adapters.getState(adapterId)) {
            case 'inquiry':
                throw new Error(`Trying to lock adapter ${adapterId} even though it is already locked`);
            case 'resetting':
                throw new Error(`Cannot lock resetting adapter ${adapterId}`);
            case 'scan':
                this.logger.debug(`Stopping scanning for BLE peripherals on adapter ${adapterId}`);
                noble_1.default.stopScanning();
        }
        this.adapters.setState(adapterId, 'inquiry');
    }
    async unlockAdapter(adapterId) {
        if (this.adapters.getState(adapterId) != 'inquiry') {
            return;
        }
        this.logger.debug(`Unlocking adapter ${adapterId}`);
        this.adapters.setState(adapterId, 'inactive');
        if (adapterId == this._lowEnergyAdapterId) {
            await this.handleAdapterStateChange(noble_1.default.state);
        }
    }
    unlockDeadlockedAdapters() {
        this.adapters.forEach(async (adapter, adapterId) => {
            if (adapter.state === 'inquiry' &&
                adapter.startedAt.getTime() < Date.now() - INQUIRY_LOCK_TIMEOUT) {
                this.logger.log(`Detected unusually long lock on Bluetooth adapter ${adapterId}, force unlocking`);
                await this.unlockAdapter(adapterId);
            }
        });
    }
    async verifyLowEnergyScanner() {
        if (this.timeSinceLastDiscovery > SCAN_NO_PERIPHERAL_TIMEOUT) {
            this.logger.warn('Did not detect any low energy advertisements in a while, resetting');
            await this.resetHciDevice(this._lowEnergyAdapterId);
        }
    }
    setupNoble() {
        this._lowEnergyAdapterId = parseInt(process.env.NOBLE_HCI_DEVICE_ID) || 0;
        this.adapters.setState(this._lowEnergyAdapterId, 'inactive');
        const debouncedScanRecovery = lodash_1.default.debounce(async () => {
            if (this.adapters.getState(this._lowEnergyAdapterId) === 'scan') {
                this.logger.debug('Trying to recover low energy scanner');
                await this.resetHciDevice(this._lowEnergyAdapterId);
            }
        }, 10 * 1000, { maxWait: 30 * 1000 });
        noble_1.default.on('stateChange', this.handleAdapterStateChange.bind(this));
        noble_1.default.on('discover', debouncedScanRecovery.cancel.bind(this));
        noble_1.default.on('discover', this.handleDiscover.bind(this));
        noble_1.default.on('scanStart', debouncedScanRecovery.cancel.bind(this));
        noble_1.default.on('scanStop', debouncedScanRecovery.bind(this));
        noble_1.default.on('scanStart', this.handleScanStart.bind(this));
        noble_1.default.on('scanStop', this.handleScanStop.bind(this));
        noble_1.default.on('warning', (message) => {
            if (message == 'unknown peripheral undefined RSSI update!') {
                return;
            }
            this.logger.warn(message);
        });
    }
    handleScanStart() {
        this.logger.debug(`Started scanning for BLE peripherals on adapter ${this._lowEnergyAdapterId}`);
        this.scanStartedAt = new Date();
        this.adapters.setState(this._lowEnergyAdapterId, 'scan');
    }
    handleScanStop() {
        this.logger.debug(`Stopped scanning for BLE peripherals on adapter ${this._lowEnergyAdapterId}`);
        this.scanStartedAt = null;
        if (this.adapters.getState(this._lowEnergyAdapterId) === 'scan') {
            this.adapters.setState(this._lowEnergyAdapterId, 'inactive');
        }
    }
    handleDiscover() {
        this.lastLowEnergyDiscovery = new Date();
        this.advertisementReceivedCounter.inc();
        const adapterState = this.adapters.get(this._lowEnergyAdapterId);
        if (adapterState.state === 'inactive' ||
            (adapterState.state === 'resetting' &&
                adapterState.startedAt.getTime() < Date.now() - 1000)) {
            this.handleScanStart();
        }
    }
    async handleAdapterStateChange(state) {
        this.logger.debug(`Adapter ${this._lowEnergyAdapterId} went into state ${state}`);
        const adapterState = this.adapters.getState(this._lowEnergyAdapterId);
        if (state === 'poweredOn') {
            if (['resetting', 'inactive'].includes(adapterState)) {
                this.logger.debug(`Starting scanning for BLE peripherals on adapter ${this._lowEnergyAdapterId}`);
                noble_1.default.startScanning([], true);
            }
        }
        else if (adapterState === 'scan') {
            this.logger.debug(`Adapter ${this._lowEnergyAdapterId} is set to inactive`);
            this.adapters.setState(this._lowEnergyAdapterId, 'inactive');
        }
    }
    async connectLowEnergyDeviceWithRetry(peripheral, tries) {
        this.logger.debug(`Connecting to BLE device at address ${peripheral.address}`);
        const cutoffTime = Date.now() + BLE_CONNECTION_TIMEOUT;
        let timeout = cutoffTime - Date.now();
        do {
            try {
                await (0, promises_1.promiseWithTimeout)(peripheral.connectAsync(), timeout);
            }
            catch (e) {
                this.logger.debug(`Connection error ${peripheral.address}: ${e})`);
            }
            if (peripheral.state === 'connecting') {
                try {
                    await (0, promises_1.promiseWithTimeout)(peripheral.disconnectAsync(), 1000);
                }
                catch (e) {
                    this.logger.debug(`Disconnect error ${peripheral.address}: ${e})`);
                }
            }
            await (0, promises_1.sleep)(100);
            tries--;
            timeout = cutoffTime - Date.now();
            this.logger.debug(`Connect attempt ${5 - tries}: State ${peripheral.state} with time remaining ${timeout} ms`);
        } while (tries > 0 && timeout > 0 && peripheral.state !== 'connected');
        if (peripheral.state !== 'connected') {
            throw new Error(timeout <= 0 ? 'timed out' : 'retries exceeded');
        }
        return peripheral;
    }
};
__decorate([
    (0, schedule_1.Interval)(10 * 1000),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], BluetoothService.prototype, "unlockDeadlockedAdapters", null);
__decorate([
    (0, schedule_1.Interval)(15 * 1000),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], BluetoothService.prototype, "verifyLowEnergyScanner", null);
BluetoothService = BluetoothService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(1, (0, nestjs_prometheus_1.InjectMetric)('bluetooth_le_advertisements_received_count')),
    __metadata("design:paramtypes", [config_service_1.ConfigService,
        prom_client_1.Counter])
], BluetoothService);
exports.BluetoothService = BluetoothService;
//# sourceMappingURL=bluetooth.service.js.map