"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var BluetoothClassicService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BluetoothClassicService = void 0;
const common_1 = require("@nestjs/common");
const config_service_1 = require("../../config/config.service");
const entities_service_1 = require("../../entities/entities.service");
const cluster_service_1 = require("../../cluster/cluster.service");
const util = __importStar(require("util"));
const child_process_1 = require("child_process");
const new_rssi_event_1 = require("./new-rssi.event");
const lodash_1 = __importDefault(require("lodash"));
const schedule_1 = require("@nestjs/schedule");
const sensor_config_1 = require("../home-assistant/sensor-config");
const device_tracker_config_1 = require("../home-assistant/device-tracker-config");
const bluetooth_classic_const_1 = require("./bluetooth-classic.const");
const room_presence_distance_sensor_1 = require("../../integration-support/room-presence/room-presence-distance.sensor");
const filters_1 = require("../../util/filters");
const id_1 = require("../../util/id");
const home_assistant_const_1 = require("../home-assistant/home-assistant.const");
const switch_1 = require("../../entities/switch");
const switch_config_1 = require("../home-assistant/switch-config");
const device_tracker_1 = require("../../entities/device-tracker");
const room_presence_proxy_1 = require("../../integration-support/room-presence/room-presence.proxy");
const bluetooth_service_1 = require("../../integration-support/bluetooth/bluetooth.service");
const execPromise = util.promisify(child_process_1.exec);
let BluetoothClassicService = BluetoothClassicService_1 = class BluetoothClassicService extends (0, filters_1.KalmanFilterable)(Object, 0.01, 0.7) {
    constructor(bluetoothService, configService, entitiesService, clusterService, schedulerRegistry) {
        super();
        this.bluetoothService = bluetoothService;
        this.configService = configService;
        this.entitiesService = entitiesService;
        this.clusterService = clusterService;
        this.schedulerRegistry = schedulerRegistry;
        this.rotationOffset = 0;
        this.deviceMap = new Map();
        this.config = this.configService.get('bluetoothClassic');
        this.logger = new common_1.Logger(BluetoothClassicService_1.name);
    }
    async onModuleInit() {
        try {
            await execPromise('hcitool -h');
        }
        catch (e) {
            throw new Error('Could not find the required hcitool command');
        }
    }
    onApplicationBootstrap() {
        this.inquiriesSwitch = this.createInquiriesSwitch();
        this.clusterService.on(bluetooth_classic_const_1.REQUEST_RSSI_CHANNEL, this.handleRssiRequest.bind(this));
        this.clusterService.on(bluetooth_classic_const_1.NEW_RSSI_CHANNEL, this.handleNewRssi.bind(this));
        this.clusterService.subscribe(bluetooth_classic_const_1.NEW_RSSI_CHANNEL);
        const interval = setInterval(this.distributeInquiries.bind(this), this.config.interval * 1000);
        this.schedulerRegistry.addInterval(`bluetooth_classic_distribute_inquiries`, interval);
    }
    async handleRssiRequest(address) {
        if (lodash_1.default.isEmpty(address)) {
            this.logger.error('RSSI update for empty address requested');
            return;
        }
        if (this.shouldInquire()) {
            let rssi;
            try {
                rssi = await this.bluetoothService.inquireClassicRssi(this.config.hciDeviceId, address);
            }
            catch (e) {
                this.logger.error(`Failed to retrieve RSSI for ${address}: ${e.message}`);
            }
            if (rssi !== undefined) {
                rssi = rssi * this.config.rssiFactor;
                rssi = lodash_1.default.round(this.filterRssi(address, rssi), 1);
                let device;
                if (this.deviceMap.has(address)) {
                    device = this.deviceMap.get(address);
                }
                else {
                    device = await this.bluetoothService.inquireClassicDeviceInfo(this.config.hciDeviceId, address);
                    this.deviceMap.set(address, device);
                }
                let minRssi;
                if (lodash_1.default.isObject(this.config.minRssi)) {
                    minRssi = lodash_1.default.defaultTo(this.config.minRssi[address], this.config.minRssi.default);
                }
                else if (lodash_1.default.isNumber(this.config.minRssi)) {
                    minRssi = this.config.minRssi;
                }
                const event = new new_rssi_event_1.NewRssiEvent(this.configService.get('global').instanceName, device, rssi, rssi < minRssi);
                this.clusterService.publish(bluetooth_classic_const_1.NEW_RSSI_CHANNEL, event);
                this.handleNewRssi(event);
            }
        }
    }
    async handleNewRssi(event) {
        var _a;
        this.logger.debug(`Received RSSI of ${event.rssi} for ${event.device.address} from ${event.instanceName}`);
        const baseId = ((_a = this.config.entityOverrides[event.device.address]) === null || _a === void 0 ? void 0 : _a.id) ||
            event.device.address;
        const sensorId = (0, id_1.makeId)(`bluetooth-classic ${baseId}`);
        let sensor;
        if (this.entitiesService.has(sensorId)) {
            sensor = this.entitiesService.get(sensorId);
        }
        else {
            sensor = await this.createSensor(sensorId, event.device);
        }
        sensor.timeout = this.calculateCurrentTimeout();
        sensor.handleNewDistance(event.instanceName, event.rssi * -1, event.outOfRange);
    }
    distributeInquiries() {
        if (this.clusterService.isMajorityLeader()) {
            const nodes = this.getParticipatingNodes();
            const addresses = [...this.config.addresses];
            if (this.rotationOffset >= Math.max(nodes.length, addresses.length)) {
                this.rotationOffset = 0;
            }
            const [nodeSubset, addressSubset] = this.mapArrays(nodes, addresses, this.rotationOffset);
            nodeSubset.forEach((node, index) => {
                if (addressSubset[index] == null) {
                    this.logger.error(`Trying to request inquiry without MAC! Current index: ${this.rotationOffset}. Addresses in this round: ${addressSubset}. Addresses overall: ${addresses}.`);
                }
                if (node.last === undefined) {
                    this.handleRssiRequest(addressSubset[index]);
                }
                else {
                    this.clusterService.send(bluetooth_classic_const_1.REQUEST_RSSI_CHANNEL, addressSubset[index], node.id);
                }
            });
            this.rotationOffset++;
        }
    }
    filterRssi(address, rssi) {
        return this.kalmanFilter(rssi, address);
    }
    updateSensorState(sensor, device) {
        if (!this.shouldInquire() && this.config.preserveState) {
            const instanceName = this.configService.get('global').instanceName;
            const previousReading = sensor.distances[instanceName];
            if (previousReading &&
                Date.now() <
                    previousReading.lastUpdatedAt.getTime() +
                        this.calculateCurrentTimeout() * 1000) {
                const event = new new_rssi_event_1.NewRssiEvent(instanceName, device, previousReading.distance * -1, previousReading.outOfRange);
                this.clusterService.publish(bluetooth_classic_const_1.NEW_RSSI_CHANNEL, event);
                this.handleNewRssi(event);
            }
        }
        else {
            sensor.updateState();
        }
    }
    getParticipatingNodes() {
        const nodes = Object.values(this.clusterService.nodes());
        return nodes.filter((node) => { var _a; return node.state !== 'removed' && ((_a = node.channels) === null || _a === void 0 ? void 0 : _a.includes(bluetooth_classic_const_1.NEW_RSSI_CHANNEL)); });
    }
    shouldInquire() {
        var _a;
        return (_a = this.inquiriesSwitch) === null || _a === void 0 ? void 0 : _a.state;
    }
    createInquiriesSwitch() {
        const instanceName = this.configService.get('global').instanceName;
        const customizations = [
            {
                for: switch_config_1.SwitchConfig,
                overrides: {
                    icon: 'mdi:bluetooth-audio',
                },
            },
        ];
        const inquiriesSwitch = this.entitiesService.add(new switch_1.Switch('bluetooth-classic-inquiries-switch', `${instanceName} Bluetooth Inquiries`), customizations);
        if (this.config.inquireFromStart) {
            inquiriesSwitch.turnOn();
        }
        else {
            inquiriesSwitch.turnOff();
        }
        return inquiriesSwitch;
    }
    async createSensor(sensorId, device) {
        var _a;
        const baseName = ((_a = this.config.entityOverrides[device.address]) === null || _a === void 0 ? void 0 : _a.name) || device.name;
        const deviceInfo = {
            identifiers: device.address,
            name: baseName,
            manufacturer: device.manufacturer,
            connections: [['mac', device.address]],
            viaDevice: home_assistant_const_1.DISTRIBUTED_DEVICE_ID,
        };
        const deviceTracker = this.createDeviceTracker((0, id_1.makeId)(`${sensorId}-tracker`), `${baseName} Tracker`, deviceInfo);
        const customizations = [
            {
                for: sensor_config_1.SensorConfig,
                overrides: {
                    icon: 'mdi:bluetooth',
                    device: deviceInfo,
                },
            },
        ];
        const rawSensor = new room_presence_distance_sensor_1.RoomPresenceDistanceSensor(sensorId, `${baseName} Room Presence`, 0);
        const sensorProxy = new Proxy(rawSensor, new room_presence_proxy_1.RoomPresenceProxyHandler(deviceTracker));
        const sensor = this.entitiesService.add(sensorProxy, customizations);
        const interval = setInterval(() => this.updateSensorState(sensor, device), this.config.interval * 1000);
        this.schedulerRegistry.addInterval(`${sensorId}_timeout_check`, interval);
        return sensor;
    }
    createDeviceTracker(id, name, deviceInfo) {
        const trackerCustomizations = [
            {
                for: device_tracker_config_1.DeviceTrackerConfig,
                overrides: {
                    sourceType: 'bluetooth',
                    device: deviceInfo,
                },
            },
        ];
        return this.entitiesService.add(new device_tracker_1.DeviceTracker(id, name, true), trackerCustomizations);
    }
    calculateCurrentTimeout() {
        const nodes = this.getParticipatingNodes();
        const addresses = this.config.addresses;
        return (Math.max(nodes.length, addresses.length) *
            this.config.timeoutCycles *
            this.config.interval);
    }
    mapArrays(a1, a2, offset) {
        const [small, large] = a1.length > a2.length ? [a2, a1] : [a1, a2];
        const largeSubset = large.slice(offset, offset + small.length);
        if (offset + small.length > large.length) {
            largeSubset.push(...large.slice(0, small.length - largeSubset.length));
        }
        return a1.length > a2.length ? [largeSubset, a2] : [a1, largeSubset];
    }
};
BluetoothClassicService = BluetoothClassicService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [bluetooth_service_1.BluetoothService,
        config_service_1.ConfigService,
        entities_service_1.EntitiesService,
        cluster_service_1.ClusterService,
        schedule_1.SchedulerRegistry])
], BluetoothClassicService);
exports.BluetoothClassicService = BluetoothClassicService;
//# sourceMappingURL=bluetooth-classic.service.js.map