/// <reference types="node" />
/// <reference types="typings/kalmanjs" />
import { OnApplicationBootstrap, OnModuleInit } from '@nestjs/common';
import { Peripheral } from '@mkerix/noble';
import { EntitiesService } from '../../entities/entities.service';
import { Tag } from './tag';
import { ConfigService } from '../../config/config.service';
import { ClusterService } from '../../cluster/cluster.service';
import { NewDistanceEvent } from './new-distance.event';
import { Device } from '../home-assistant/device';
import { SchedulerRegistry } from '@nestjs/schedule';
import { DeviceTracker } from '../../entities/device-tracker';
import { Sensor } from '../../entities/sensor';
import { BluetoothLowEnergyPresenceSensor } from './bluetooth-low-energy-presence.sensor';
import { BluetoothService } from '../../integration-support/bluetooth/bluetooth.service';
export declare const NEW_DISTANCE_CHANNEL = "bluetooth-low-energy.new-distance";
declare const BluetoothLowEnergyService_base: {
    new (...args: any[]): {
        kalmanFilterMap: Map<string, import("kalmanjs").default>;
        kalmanFilter(value: number, id?: string): number;
    };
} & ObjectConstructor;
export declare class BluetoothLowEnergyService extends BluetoothLowEnergyService_base implements OnModuleInit, OnApplicationBootstrap {
    private readonly bluetoothService;
    private readonly entitiesService;
    private readonly configService;
    private readonly clusterService;
    private readonly schedulerRegistry;
    private readonly config;
    private readonly logger;
    private readonly loggedIds;
    private readonly companionAppTags;
    private readonly companionAppDenylist;
    private tagUpdaters;
    constructor(bluetoothService: BluetoothService, entitiesService: EntitiesService, configService: ConfigService, clusterService: ClusterService, schedulerRegistry: SchedulerRegistry);
    onModuleInit(): void;
    onApplicationBootstrap(): void;
    handleDiscovery(peripheral: Peripheral): Promise<void>;
    discoverCompanionAppId(tag: Tag): Promise<string | null>;
    handleNewDistance(event: NewDistanceEvent): void;
    isIBeacon(manufacturerData: Buffer): boolean;
    isAllowlistEnabled(): boolean;
    isDenylistEnabled(): boolean;
    isOnAllowlist(id: string): boolean;
    isOnDenylist(id: string): boolean;
    filterRssi(tagId: string, rssi: number): number;
    protected createRoomPresenceSensor(sensorId: string, deviceId: string, deviceName: string, hasBattery: boolean): BluetoothLowEnergyPresenceSensor;
    protected createDeviceTracker(id: string, name: string, deviceInfo: Device): DeviceTracker;
    protected createBatterySensor(id: string, name: string, deviceInfo: Device): Sensor;
    protected createTag(peripheral: Peripheral): Tag;
    protected applyOverrides(tag: Tag): Tag;
    protected applyCompanionAppOverride(tag: Tag): Promise<Tag>;
    protected handleAppDiscovery(tagId: string, appId: string): void;
    private banDeviceFromDiscovery;
    private static isAppleDevice;
    private static extractOverflowArea;
    private static overflowContainsCompanionApp;
}
export {};
