"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var GridEyeService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.GridEyeService = void 0;
const common_1 = require("@nestjs/common");
const i2c_bus_1 = __importDefault(require("i2c-bus"));
const entities_service_1 = require("../../entities/entities.service");
const sensor_1 = require("../../entities/sensor");
const math = __importStar(require("mathjs"));
const schedule_1 = require("@nestjs/schedule");
const thermopile_occupancy_service_1 = require("../../integration-support/thermopile/thermopile-occupancy.service");
const config_service_1 = require("../../config/config.service");
const sensor_config_1 = require("../home-assistant/sensor-config");
const camera_1 = require("../../entities/camera");
const async_mutex_1 = require("async-mutex");
const TEMPERATURE_REGISTER_START = 0x80;
const FRAMERATE_REGISTER = 0x02;
let GridEyeService = GridEyeService_1 = class GridEyeService extends thermopile_occupancy_service_1.ThermopileOccupancyService {
    constructor(entitiesService, configService) {
        super();
        this.entitiesService = entitiesService;
        this.configService = configService;
        this.heatmapMutex = new async_mutex_1.Mutex();
        this.config = this.configService.get('gridEye');
        this.logger = new common_1.Logger(GridEyeService_1.name);
    }
    async onApplicationBootstrap() {
        this.logger.log(`Opening i2c bus ${this.config.busNumber}`);
        this.i2cBus = await i2c_bus_1.default.openPromisified(this.config.busNumber);
        this.setRegister(FRAMERATE_REGISTER, 1);
        this.sensor = this.createSensor();
        if (this.config.heatmap.enabled) {
            if (this.isHeatmapAvailable()) {
                this.camera = this.createHeatmapCamera();
            }
            else {
                this.logger.error("Heatmaps cannot be rendered since the required dependencies aren't available. More info: https://www.room-assistant.io/integrations/grid-eye.html#requirements");
            }
        }
    }
    async onApplicationShutdown() {
        this.logger.log(`Closing i2c bus ${this.config.busNumber}`);
        return this.i2cBus.close();
    }
    async updateState() {
        const temperatures = await this.getPixelTemperatures();
        const coordinates = await this.getCoordinates(temperatures, this.config.deltaThreshold);
        this.sensor.state = coordinates.length;
        this.sensor.attributes.coordinates = coordinates;
        if (this.camera != undefined && !this.heatmapMutex.isLocked()) {
            await this.heatmapMutex.runExclusive(async () => {
                this.camera.state = await this.generateHeatmap(temperatures, this.config.heatmap);
            });
        }
    }
    async getPixelTemperatures() {
        const temperatures = [];
        for (let i = 0; i < 64; i++) {
            temperatures.push(await this.getPixelTemperature(i));
        }
        let grid = math.reshape(temperatures, [8, 8]);
        if (this.config.maskZeroBasedValues) {
            grid = await this.maskZeroBasedValues(grid);
        }
        return grid;
    }
    async maskZeroBasedValues(temperatures) {
        const correctedMean = math.mean(temperatures.flat().filter((v) => v >= 1 || v < 0));
        return math
            .matrix(temperatures)
            .map((v) => (v >= 1 || v < 0 ? v : correctedMean))
            .toArray();
    }
    async getPixelTemperature(pixelAddr) {
        const pixelLowRegister = TEMPERATURE_REGISTER_START + 2 * pixelAddr;
        let temperature = await this.getRegister(pixelLowRegister, 2);
        if (temperature & (1 << 11)) {
            temperature &= ~(1 << 11);
            temperature = temperature * -1;
        }
        return temperature * 0.25;
    }
    async getRegister(register, length) {
        const commandBuffer = Buffer.alloc(1);
        commandBuffer.writeUInt8(register, 0);
        const resultBuffer = Buffer.alloc(length);
        await this.i2cBus.i2cWrite(this.config.address, commandBuffer.length, commandBuffer);
        await this.i2cBus.i2cRead(this.config.address, length, resultBuffer);
        const lsb = resultBuffer.readUInt8(0);
        const msb = resultBuffer.readUInt8(1);
        return (msb << 8) | lsb;
    }
    async setRegister(register, value) {
        const commandBuffer = Buffer.alloc(2);
        commandBuffer.writeUInt8(register, 0);
        commandBuffer.writeUInt8(value, 1);
        await this.i2cBus.i2cWrite(this.config.address, commandBuffer.length, commandBuffer);
    }
    createSensor() {
        const customizations = [
            {
                for: sensor_config_1.SensorConfig,
                overrides: {
                    icon: 'mdi:account',
                    unitOfMeasurement: 'person',
                },
            },
        ];
        return this.entitiesService.add(new sensor_1.Sensor('grideye_occupancy_count', 'GridEYE Occupancy Count'), customizations);
    }
    createHeatmapCamera() {
        return this.entitiesService.add(new camera_1.Camera('grideye_heatmap', 'GridEYE Heatmap'));
    }
};
__decorate([
    (0, schedule_1.Interval)(1000),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], GridEyeService.prototype, "updateState", null);
GridEyeService = GridEyeService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [entities_service_1.EntitiesService,
        config_service_1.ConfigService])
], GridEyeService);
exports.GridEyeService = GridEyeService;
//# sourceMappingURL=grid-eye.service.js.map