"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var MqttService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MqttService = void 0;
const common_1 = require("@nestjs/common");
const config_service_1 = require("../../config/config.service");
const entities_service_1 = require("../../entities/entities.service");
const nest_emitter_1 = require("nest-emitter");
const async_mqtt_1 = __importDefault(require("async-mqtt"));
const id_1 = require("../../util/id");
const lodash_1 = __importDefault(require("lodash"));
const cluster_service_1 = require("../../cluster/cluster.service");
let MqttService = MqttService_1 = class MqttService {
    constructor(configService, entitiesService, clusterService, emitter) {
        this.configService = configService;
        this.entitiesService = entitiesService;
        this.clusterService = clusterService;
        this.emitter = emitter;
        this.logger = new common_1.Logger(MqttService_1.name);
        this.config = configService.get('mqtt');
    }
    async onModuleInit() {
        try {
            this.mqttClient = await async_mqtt_1.default.connectAsync(this.config.mqttUrl, { ...this.config.mqttOptions }, false);
            this.mqttClient.on('error', (e) => this.logger.error(e.message, e.stack));
            this.mqttClient.on('connect', this.handleReconnect.bind(this));
            this.logger.log(`Successfully connected to MQTT broker at ${this.config.mqttUrl}`);
            this.emitter.on('entityUpdate', this.handleEntityUpdate.bind(this));
        }
        catch (e) {
            this.logger.error(e.message, e.stack);
        }
        this.clusterService.on('elected', this.refreshEntities.bind(this));
    }
    isConnected() {
        var _a;
        return (_a = this.mqttClient) === null || _a === void 0 ? void 0 : _a.connected;
    }
    refreshEntities() {
        this.logger.log('Refreshing entity states');
        this.entitiesService.getAll().forEach((entity) => {
            this.handleEntityUpdate(entity, undefined, this.entitiesService.hasAuthorityOver(entity));
        });
    }
    handleEntityUpdate(entity, diff, hasAuthority) {
        const message = {
            entity,
            diff,
            hasAuthority,
        };
        this.mqttClient.publish(this.getTopicName(entity), JSON.stringify(message), {
            qos: this.config.qos,
            retain: this.config.retain,
        });
    }
    handleReconnect() {
        this.logger.log('Re-connected to broker');
        this.refreshEntities();
    }
    getTopicName(entity) {
        return `${this.config.baseTopic}/${(0, id_1.makeId)(this.configService.get('global').instanceName)}/${lodash_1.default.kebabCase(entity.constructor.name)}/${(0, id_1.makeId)(entity.id)}`;
    }
};
MqttService = MqttService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(3, (0, nest_emitter_1.InjectEventEmitter)()),
    __metadata("design:paramtypes", [config_service_1.ConfigService,
        entities_service_1.EntitiesService,
        cluster_service_1.ClusterService, Object])
], MqttService);
exports.MqttService = MqttService;
//# sourceMappingURL=mqtt.service.js.map