"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var OmronD6tService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.OmronD6tService = void 0;
const common_1 = require("@nestjs/common");
const entities_service_1 = require("../../entities/entities.service");
const config_service_1 = require("../../config/config.service");
const i2c_bus_1 = __importDefault(require("i2c-bus"));
const schedule_1 = require("@nestjs/schedule");
const math = __importStar(require("mathjs"));
const sensor_1 = require("../../entities/sensor");
const i2c_error_1 = require("./i2c.error");
const sensor_config_1 = require("../home-assistant/sensor-config");
const thermopile_occupancy_service_1 = require("../../integration-support/thermopile/thermopile-occupancy.service");
const camera_1 = require("../../entities/camera");
const async_mutex_1 = require("async-mutex");
const TEMPERATURE_COMMAND = 0x4c;
let OmronD6tService = OmronD6tService_1 = class OmronD6tService extends thermopile_occupancy_service_1.ThermopileOccupancyService {
    constructor(entitiesService, configService) {
        super();
        this.entitiesService = entitiesService;
        this.configService = configService;
        this.heatmapMutex = new async_mutex_1.Mutex();
        this.config = this.configService.get('omronD6t');
        this.logger = new common_1.Logger(OmronD6tService_1.name);
    }
    async onApplicationBootstrap() {
        this.logger.log(`Opening i2c bus ${this.config.busNumber}`);
        this.i2cBus = await i2c_bus_1.default.openPromisified(this.config.busNumber);
        this.sensor = this.createSensor();
        if (this.config.heatmap.enabled) {
            if (this.isHeatmapAvailable()) {
                this.camera = this.createHeatmapCamera();
            }
            else {
                this.logger.error("Heatmaps cannot be rendered since the required dependencies aren't available. More info: https://www.room-assistant.io/integrations/omron-d6t.html#requirements");
            }
        }
    }
    async onApplicationShutdown() {
        this.logger.log(`Closing i2c bus ${this.config.busNumber}`);
        return this.i2cBus.close();
    }
    async updateState() {
        try {
            const temperatures = await this.getPixelTemperatures();
            const coordinates = await this.getCoordinates(temperatures, this.config.deltaThreshold);
            this.sensor.state = coordinates.length;
            this.sensor.attributes.coordinates = coordinates;
            if (this.camera != undefined && !this.heatmapMutex.isLocked()) {
                await this.heatmapMutex.runExclusive(async () => {
                    this.camera.state = await this.generateHeatmap(temperatures, this.config.heatmap);
                });
            }
        }
        catch (e) {
            if (e instanceof i2c_error_1.I2CError) {
                this.logger.debug(`Error during I2C communication: ${e.message}`);
            }
            else {
                this.logger.error(`Retrieving the state from D6T sensor failed: ${e.message}`, e.stack);
            }
        }
    }
    async getPixelTemperatures() {
        const commandBuffer = Buffer.alloc(1);
        const resultBuffer = Buffer.alloc(35);
        commandBuffer.writeUInt8(TEMPERATURE_COMMAND, 0);
        await this.i2cBus.i2cWrite(this.config.address, commandBuffer.length, commandBuffer);
        await this.i2cBus.i2cRead(this.config.address, resultBuffer.length, resultBuffer);
        if (!this.checkPEC(resultBuffer, 34)) {
            throw new i2c_error_1.I2CError('PEC check for the message failed');
        }
        const pixelTemperatures = [];
        for (let i = 2; i < resultBuffer.length - 1; i += 2) {
            const temperature = (256 * resultBuffer.readUInt8(i + 1) + resultBuffer.readUInt8(i)) / 10;
            pixelTemperatures.push(temperature);
        }
        return math.reshape(pixelTemperatures, [4, 4]);
    }
    checkPEC(buffer, pecIndex) {
        let crc = this.calculateCRC(0x15);
        for (let i = 0; i < pecIndex; i++) {
            crc = this.calculateCRC(buffer.readUInt8(i) ^ crc);
        }
        return crc === buffer.readUInt8(pecIndex);
    }
    calculateCRC(data) {
        const crc = new Uint8Array([data]);
        for (let i = 0; i < 8; i++) {
            const temp = crc[0];
            crc[0] = crc[0] << 1;
            if (temp & 0x80) {
                crc[0] = crc[0] ^ 0x07;
            }
        }
        return crc[0];
    }
    createSensor() {
        const customizations = [
            {
                for: sensor_config_1.SensorConfig,
                overrides: {
                    icon: 'mdi:account',
                    unitOfMeasurement: 'person',
                },
            },
        ];
        return this.entitiesService.add(new sensor_1.Sensor('d6t_occupancy_count', 'D6T Occupancy Count'), customizations);
    }
    createHeatmapCamera() {
        return this.entitiesService.add(new camera_1.Camera('d6t_heatmap', 'D6T Heatmap'));
    }
};
__decorate([
    (0, schedule_1.Interval)(250),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], OmronD6tService.prototype, "updateState", null);
OmronD6tService = OmronD6tService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [entities_service_1.EntitiesService,
        config_service_1.ConfigService])
], OmronD6tService);
exports.OmronD6tService = OmronD6tService;
//# sourceMappingURL=omron-d6t.service.js.map