"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var ShellService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShellService = void 0;
const common_1 = require("@nestjs/common");
const config_service_1 = require("../../config/config.service");
const entities_service_1 = require("../../entities/entities.service");
const schedule_1 = require("@nestjs/schedule");
const util = __importStar(require("util"));
const child_process_1 = require("child_process");
const id_1 = require("../../util/id");
const sensor_1 = require("../../entities/sensor");
const sensor_config_1 = require("../home-assistant/sensor-config");
const cron_1 = require("cron");
const switch_config_1 = require("../home-assistant/switch-config");
const shell_switch_1 = require("./shell.switch");
let ShellService = ShellService_1 = class ShellService {
    constructor(configService, entitiesService, schedulerRegistry) {
        this.configService = configService;
        this.entitiesService = entitiesService;
        this.schedulerRegistry = schedulerRegistry;
        this.config = this.configService.get('shell');
        this.logger = new common_1.Logger(ShellService_1.name);
    }
    onApplicationBootstrap() {
        this.config.sensors.forEach((sensorOptions) => {
            const sensor = this.createSensor(sensorOptions.name, sensorOptions.deviceClass, sensorOptions.unitOfMeasurement, sensorOptions.icon);
            const regex = sensorOptions.regex
                ? new RegExp(sensorOptions.regex)
                : undefined;
            const job = new cron_1.CronJob(sensorOptions.cron, async () => {
                sensor.state = await this.executeCommand(sensorOptions.command, regex);
            });
            this.schedulerRegistry.addCronJob((0, id_1.makeId)(`shell ${sensorOptions.name}`), job);
            job.start();
        });
        this.config.switches.forEach((switchOptions) => {
            this.createSwitch(switchOptions.name, switchOptions.onCommand, switchOptions.offCommand, switchOptions.icon);
        });
    }
    async executeCommand(command, regex) {
        const execPromise = util.promisify(child_process_1.exec);
        const output = await execPromise(command);
        this.logger.debug(`${command} returned stdout:\n${output.stdout}\nstderr:\n${output.stderr}`);
        if (regex) {
            const results = regex.exec(output.stdout);
            return results ? results[1] || results[0] : undefined;
        }
        else {
            return output.stdout.trim();
        }
    }
    createSensor(name, deviceClass, unitOfMeasurement, icon) {
        const id = (0, id_1.makeId)(`shell ${name}`);
        const customizations = [
            {
                for: sensor_config_1.SensorConfig,
                overrides: {
                    deviceClass,
                    unitOfMeasurement,
                    icon,
                },
            },
        ];
        return this.entitiesService.add(new sensor_1.Sensor(id, name), customizations);
    }
    createSwitch(name, onCommand, offCommand, icon) {
        const id = (0, id_1.makeId)(`shell ${name}`);
        const customizations = [
            {
                for: switch_config_1.SwitchConfig,
                overrides: {
                    icon,
                },
            },
        ];
        return this.entitiesService.add(new shell_switch_1.ShellSwitch(id, name, onCommand, offCommand), customizations);
    }
};
ShellService = ShellService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [config_service_1.ConfigService,
        entities_service_1.EntitiesService,
        schedule_1.SchedulerRegistry])
], ShellService);
exports.ShellService = ShellService;
//# sourceMappingURL=shell.service.js.map