/// <reference types="node" />
export declare const enum EventType {
    temperature = 4100,
    humidity = 4102,
    illuminance = 4103,
    moisture = 4104,
    fertility = 4105,
    battery = 4106,
    temperatureAndHumidity = 4109
}
export declare class Event {
    type: EventType;
    value: number;
}
export declare class ServiceData {
    frameControl: FrameControl;
    protocolVersion: number;
    productId: number;
    productName: string;
    frameCounter: number;
    macAddress: string;
    capabilities: Capabilities;
    eventType?: EventType;
    eventLength?: number;
    events: Event[];
}
export declare const enum ProductId {
    HHCCJCV01 = 152,
    LYWSD03MMC = 1371,
    LYWSD02 = 1115,
    LYWSDCGQ = 426
}
declare class FrameControl {
    isFactoryNew: boolean;
    isConnected: boolean;
    isCentral: boolean;
    isEncrypted: boolean;
    hasMacAddress: boolean;
    hasCapabilities: boolean;
    hasEvent: boolean;
    hasCustomData: boolean;
    hasSubtitle: boolean;
    hasBinding: boolean;
}
declare class Capabilities {
    connectable: boolean;
    central: boolean;
    secure: boolean;
    io: boolean;
}
export declare class Parser {
    private buffer;
    private readonly bindKey?;
    private baseByteLength;
    private frameControl;
    private eventType;
    constructor(buffer: Buffer, bindKey?: string | null);
    parse(): ServiceData;
    parseFrameControl(): FrameControl;
    parseProtocolVersion(): number;
    parseProductId(): number;
    parseProductName(): string;
    parseFrameCounter(): number;
    parseMacAddress(): string;
    get capabilityOffset(): number;
    parseCapabilities(): Capabilities;
    get eventOffset(): number;
    parseEventType(): EventType | null;
    parseEventLength(): number | null;
    decryptPayload(): void;
    parseEventData(): Event[] | null;
    parseTemperatureEvent(): Event[];
    parseHumidityEvent(): Event[];
    parseBatteryEvent(): Event[];
    parseTemperatureAndHumidityEvent(): Event[];
    parseIlluminanceEvent(): Event[];
    parseFertilityEvent(): Event[];
    parseMoistureEvent(): Event[];
    toString(): string;
}
export {};
