"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WINSTON_LOGGER = void 0;
const nest_winston_1 = require("nest-winston");
const winston_1 = __importDefault(require("winston"));
const config_1 = __importDefault(require("config"));
const config = config_1.default.get('logger');
const instanceName = config_1.default.get('global.instanceName');
const missingDeps = [];
const transports = [
    new winston_1.default.transports.Console({
        format: winston_1.default.format.combine(winston_1.default.format.timestamp(), process.stdout.isTTY
            ? winston_1.default.format.colorize()
            : winston_1.default.format.uncolorize(), winston_1.default.format.printf((info) => {
            return `${new Date(info.timestamp).toLocaleString()} - ${info.level} - ${info.context}: ${info.message}`;
        })),
    }),
];
if (config.elasticsearch.enabled) {
    try {
        const ElasticsearchTransport = require('winston-elasticsearch')
            .ElasticsearchTransport;
        transports.push(new ElasticsearchTransport({
            indexPrefix: config.elasticsearch.indexPrefix,
            clientOpts: {
                node: config.elasticsearch.node,
                auth: config.elasticsearch.auth,
            },
        }));
    }
    catch (e) {
        missingDeps.push('elasticsearch');
    }
}
if (config.loki.enabled) {
    const lokiFormatter = winston_1.default.format((info) => {
        info.labels = info.labels || {};
        info.labels.context = info.context;
        info.labels.instanceName = info.instanceName;
        return info;
    });
    try {
        const LokiTransport = require('winston-loki');
        transports.push(new LokiTransport({
            host: config.loki.host,
            labels: {
                job: 'room-assistant',
            },
            format: winston_1.default.format.combine(lokiFormatter(), winston_1.default.format.printf((info) => `${info.message}`)),
            gracefulShutdown: false,
            clearOnError: true,
        }));
    }
    catch (e) {
        missingDeps.push('loki');
    }
}
exports.WINSTON_LOGGER = nest_winston_1.WinstonModule.createLogger({
    level: process.env.NODE_LOG_LEVEL || 'info',
    defaultMeta: {
        instanceName,
    },
    transports,
});
missingDeps.forEach((dep) => {
    exports.WINSTON_LOGGER.error(`Logging with ${dep} was enabled, but the dependency is missing. Please install it with "npm install -g winston-${dep}".`, null, 'LoggerService');
});
//# sourceMappingURL=logger.js.map