"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("./env");
const core_1 = require("@nestjs/core");
const app_module_1 = require("./app.module");
const logger_1 = require("./logger");
const swagger_1 = require("@nestjs/swagger");
const config_service_1 = require("./config/config.service");
const platform_ws_1 = require("@nestjs/platform-ws");
const pkg = require('../package.json');
async function bootstrap() {
    const app = await core_1.NestFactory.create(app_module_1.AppModule, {
        logger: logger_1.WINSTON_LOGGER,
    });
    app.enableShutdownHooks();
    app.useWebSocketAdapter(new platform_ws_1.WsAdapter(app));
    const options = new swagger_1.DocumentBuilder()
        .setTitle('room-assistant')
        .setDescription('Presence tracking and more for automation on the room-level.')
        .setVersion(pkg.version)
        .setExternalDoc('Website', 'https://room-assistant.io')
        .setLicense('MIT License', 'https://github.com/mKeRix/room-assistant/blob/main/LICENSE')
        .addTag('entities', 'Access to the internal entity registry', {
        description: 'Configuration',
        url: 'https://www.room-assistant.io/guide/entities.html',
    })
        .addTag('status', 'Status information about the room-assistant instance')
        .build();
    const document = swagger_1.SwaggerModule.createDocument(app, options);
    swagger_1.SwaggerModule.setup('api', app, document);
    const configService = app.get(config_service_1.ConfigService);
    await app.listen(configService.get('global').apiPort);
}
bootstrap();
//# sourceMappingURL=main.js.map